@@ -170,6 +170,7 @@ public:
 	                                 , const Glib::ustring & pattern
 	                                 ) ;
 	static Glib::ustring trim( const Glib::ustring & src, const Glib::ustring & c = " \t\r\n" ) ;
+	static Glib::ustring trim_trailing_new_line(const Glib::ustring& src);
 	static Glib::ustring last_line( const Glib::ustring & src );
 	static Glib::ustring get_lang() ;
 	static void tokenize( const Glib::ustring& str,
@@ -184,6 +185,7 @@ public:
 	                                         Byte_Value & fs_size, Byte_Value & fs_free,
 	                                         Glib::ustring & error_message ) ;
 	static bool is_dev_busy(const Glib::ustring& path);
+	static const Glib::ustring& first_directory(const std::vector<Glib::ustring>& paths);
 	static Byte_Value floor_size( Byte_Value value, Byte_Value rounding_size ) ;
 	static Byte_Value ceil_size( Byte_Value value, Byte_Value rounding_size ) ;
 
