@@ -28,7 +28,10 @@ class f2fs : public FileSystem
 {
 public:
 	FS get_filesystem_support() ;
+	void set_used_sectors(Partition & partition);
 	bool create( const Partition & new_partition, OperationDetail & operationdetail ) ;
+	bool resize(const Partition & partition_new, OperationDetail & operationdetail, bool fill_partition);
+	bool check_repair(const Partition & partition, OperationDetail & operationdetail);
 };
 
 } //GParted
