@@ -25,12 +25,13 @@ namespace GParted
 // The single progress bar for the current operation
 static ProgressBar single_progressbar;
 
-OperationDetail::OperationDetail() : cancelflag( 0 ), status( STATUS_NONE ), time_start( -1 ), time_elapsed( -1 )
+OperationDetail::OperationDetail() : cancelflag( 0 ), status( STATUS_NONE ), time_start( -1 ), time_elapsed( -1 ),
+                                     no_more_children( false )
 {
 }
 
 OperationDetail::OperationDetail( const Glib::ustring & description, OperationDetailStatus status, Font font ) :
-	cancelflag( 0 ), status( STATUS_NONE ), time_start( -1 ), time_elapsed( -1 )
+	cancelflag( 0 ), status( STATUS_NONE ), time_start( -1 ), time_elapsed( -1 ), no_more_children( false )
 {
 	set_description( description, font );
 	set_status( status );
@@ -74,11 +75,13 @@ void OperationDetail::set_description( const Glib::ustring & description, Font f
 	on_update( *this ) ;
 }
 
-Glib::ustring OperationDetail::get_description() const
+
+const Glib::ustring& OperationDetail::get_description() const
 {
 	return description ;
 }
-	
+
+
 void OperationDetail::set_status( OperationDetailStatus status ) 
 {	
 	if ( this ->status != STATUS_ERROR )
@@ -90,7 +93,8 @@ void OperationDetail::set_status( OperationDetailStatus status )
 				time_start = std::time( NULL ) ;
 				break ;
 			case STATUS_ERROR:
-			case STATUS_SUCCES:
+			case STATUS_WARNING:
+			case STATUS_SUCCESS:
 				if( time_start != -1 )
 					time_elapsed = std::time( NULL ) - time_start ;
 				break ;
@@ -104,6 +108,13 @@ void OperationDetail::set_status( OperationDetailStatus status )
 	}
 }
 
+void OperationDetail::set_success_and_capture_errors( bool success )
+{
+	set_status( success ? STATUS_SUCCESS : STATUS_ERROR );
+	signal_capture_errors.emit( *this, success );
+	no_more_children = true;
+}
+
 OperationDetailStatus OperationDetail::get_status() const
 {
 	return status ;
@@ -114,7 +125,8 @@ void OperationDetail::set_treepath( const Glib::ustring & treepath )
 	this ->treepath = treepath ;
 }
 
-Glib::ustring OperationDetail::get_treepath() const
+
+const Glib::ustring& OperationDetail::get_treepath() const
 {
 	return treepath ;
 }
@@ -127,19 +139,27 @@ Glib::ustring OperationDetail::get_elapsed_time() const
 	return "" ;
 }
 
-void OperationDetail::add_child( const OperationDetail & operationdetail ) 
+void OperationDetail::add_child( const OperationDetail & operationdetail )
 {
-	sub_details .push_back( new OperationDetail(operationdetail) );
-
-	sub_details.back()->set_treepath( treepath + ":" + Utils::num_to_str( sub_details .size() -1 ) );
-	sub_details.back()->signal_update.connect( sigc::mem_fun( this, &OperationDetail::on_update ) );
-	sub_details.back()->cancelconnection = signal_cancel.connect(
-				sigc::mem_fun( sub_details.back(), &OperationDetail::cancel ) );
-	if ( cancelflag )
-		sub_details.back()->cancel( cancelflag == 2 );
-	on_update( *sub_details.back() );
+	if ( no_more_children )
+		// Adding a child after this OperationDetail has been set to prevent it is
+		// a programming bug.  However the best way to report it is by adding yet
+		// another child containing the bug report, and allowing the child to be
+		// added anyway.
+		add_child_implement( OperationDetail( Glib::ustring( _("GParted Bug") ) + ": " +
+		                                      /* TO TRANSLATORS:
+		                                       * means that GParted has encountered a programming bug.  More
+		                                       * information about a step is being added after the step was
+		                                       * marked as complete.  This bug description as well as the
+		                                       * information being added will be visible in the details of the
+		                                       * applied operations.
+		                                       */
+		                                      _("Adding more information to the results of this step after it "
+		                                        "has been marked as completed"),
+		                                      STATUS_WARNING, FONT_ITALIC ) );
+	add_child_implement( operationdetail );
 }
-	
+
 std::vector<OperationDetail*> & OperationDetail::get_childs()
 {
 	return sub_details ;
@@ -178,6 +198,20 @@ void OperationDetail::stop_progressbar()
 
 // Private methods
 
+void OperationDetail::add_child_implement( const OperationDetail & operationdetail )
+{
+	sub_details .push_back( new OperationDetail( operationdetail ) );
+
+	sub_details.back()->set_treepath( treepath + ":" + Utils::num_to_str( sub_details .size() - 1 ) );
+	sub_details.back()->signal_update.connect( sigc::mem_fun( this, &OperationDetail::on_update ) );
+	sub_details.back()->cancelconnection = signal_cancel.connect(
+				sigc::mem_fun( sub_details.back(), &OperationDetail::cancel ) );
+	if ( cancelflag )
+		sub_details.back()->cancel( cancelflag == 2 );
+	sub_details.back()->signal_capture_errors.connect( this->signal_capture_errors );
+	on_update( *sub_details.back() );
+}
+
 void OperationDetail::on_update( const OperationDetail & operationdetail ) 
 {
 	if ( ! treepath .empty() )
