@@ -30,6 +30,7 @@
 #include <glibmm/ustring.h>
 #include <glibmm/stringutils.h>
 #include <glibmm/shell.h>
+#include <glibmm/fileutils.h>
 #include <gtkmm/main.h>
 #include <gtkmm/enums.h>
 #include <gtkmm/stock.h>
@@ -808,6 +809,17 @@ Glib::ustring Utils::trim( const Glib::ustring & src, const Glib::ustring & c /*
 	return src.substr(p1, (p2-p1)+1);
 }
 
+
+// Return string with optional trailing new line character removed.
+Glib::ustring Utils::trim_trailing_new_line(const Glib::ustring& src)
+{
+	Glib::ustring::size_type len = src.length();
+	if (len > 0 && src[len-1] == '\n')
+		len --;
+	return src.substr(0, len);
+}
+
+
 // Return portion of string after the last carriage return character or
 // the whole string when there is no carriage return character.
 Glib::ustring Utils::last_line( const Glib::ustring & src )
@@ -943,6 +955,18 @@ bool Utils::is_dev_busy(const Glib::ustring& path)
 }
 
 
+// Return the first path that is a directory, or the empty string.
+const Glib::ustring& Utils::first_directory(const std::vector<Glib::ustring>& paths)
+{
+	for (unsigned int i = 0; i < paths.size(); i++)
+		if (file_test(paths[i], Glib::FILE_TEST_IS_DIR))
+			return paths[i];
+
+	static const Glib::ustring not_found;
+	return not_found;
+}
+
+
 //Round down to multiple of rounding_size
 Byte_Value Utils::floor_size( Byte_Value value, Byte_Value rounding_size )
 {
