@@ -33,6 +33,8 @@
  */
 
 
+#include "common.h"
+#include "insertion_operators.h"
 #include "GParted_Core.h"
 #include "FileSystem.h"
 #include "OperationDetail.h"
@@ -62,60 +64,6 @@ namespace GParted
 {
 
 
-// Hacky XML parser which strips italic and bold markup added in
-// OperationDetail::set_description() and reverts just these 5 characters &<>'" encoded by
-// Glib::Markup::escape_text() -> g_markup_escape_text() -> append_escaped_text().
-Glib::ustring strip_markup(const Glib::ustring& str)
-{
-	size_t len = str.length();
-	size_t i = 0;
-	Glib::ustring ret;
-	ret.reserve(len);
-	while (i < len)
-	{
-		if (str.compare(i, 3, "<i>") == 0)
-			i += 3;
-		else if (str.compare(i, 4, "</i>") == 0)
-			i += 4;
-		else if (str.compare(i, 3, "<b>") == 0)
-			i += 3;
-		else if (str.compare(i, 4, "</b>") == 0)
-			i += 4;
-		else if (str.compare(i, 5, "&amp;") == 0)
-		{
-			ret.push_back('&');
-			i += 5;
-		}
-		else if (str.compare(i, 4, "&lt;") == 0)
-		{
-			ret.push_back('<');
-			i += 4;
-		}
-		else if (str.compare(i, 4, "&gt;") == 0)
-		{
-			ret.push_back('>');
-			i += 4;
-		}
-		else if (str.compare(i, 6, "&apos;") == 0)
-		{
-			ret.push_back('\'');
-			i += 6;
-		}
-		else if (str.compare(i, 6, "&quot;") == 0)
-		{
-			ret.push_back('"');
-			i += 6;
-		}
-		else
-		{
-			ret.push_back(str[i]);
-			i++;
-		}
-	}
-	return ret;
-}
-
-
 // Print method for the messages in a Partition object.
 std::ostream& operator<<(std::ostream& out, const Partition& partition)
 {
@@ -127,42 +75,6 @@ std::ostream& operator<<(std::ostream& out, const Partition& partition)
 }
 
 
-// Print method for OperationDetailStatus.
-std::ostream& operator<<(std::ostream& out, const OperationDetailStatus od_status)
-{
-	switch (od_status)
-	{
-		case STATUS_NONE:     out << "NONE";     break;
-		case STATUS_EXECUTE:  out << "EXECUTE";  break;
-		case STATUS_SUCCESS:  out << "SUCCESS";  break;
-		case STATUS_ERROR:    out << "ERROR";    break;
-		case STATUS_INFO:     out << "INFO";     break;
-		case STATUS_WARNING:  out << "WARNING";  break;
-		default:                                 break;
-	}
-	return out;
-}
-
-
-// Print method for an OperationDetail object.
-std::ostream& operator<<(std::ostream& out, const OperationDetail& od)
-{
-	out << strip_markup(od.get_description());
-	Glib::ustring elapsed = od.get_elapsed_time();
-	if (! elapsed.empty())
-		out << "    " << elapsed;
-	if (od.get_status() != STATUS_NONE)
-		out << "  (" << od.get_status() << ")";
-	out << "\n";
-
-	for (unsigned int i = 0; i < od.get_childs().size(); i++)
-	{
-		out << *od.get_childs()[i];
-	}
-	return out;
-}
-
-
 // Printable file system type which meets the requirements for a Google Test name.
 // Use GParted's file system names except when they contains any non-alphabetic chars.
 // Reference:
@@ -228,7 +140,18 @@ const std::string param_fsname(const ::testing::TestParamInfo<FSType>& info)
 	}
 
 
-const Byte_Value IMAGESIZE_Default = 256*MEBIBYTE;
+// Temporarily workaround bugs in mkfs.reiser4 and debugfs.reiser4 which occasionally
+// create and read a null UUID respectively.
+#define IGNORE_REISER4_NULL_UUID()                                            \
+	if (m_fstype == FS_REISER4 && m_partition.uuid.size() == 0)           \
+	{                                                                     \
+		std::cout << __FILE__ << ":" << __LINE__ << ": Ignore test "  \
+		          << "failure of a null UUID." << std::endl;          \
+		m_partition.uuid = "XXXXXXXXX";                               \
+	}
+
+
+const Byte_Value IMAGESIZE_Default = 320*MEBIBYTE;
 const Byte_Value IMAGESIZE_Larger  = 512*MEBIBYTE;
 
 
@@ -238,7 +161,7 @@ protected:
 	SupportedFileSystemsTest();
 
 	virtual void SetUp();
-	virtual void extra_setup(Byte_Value size = IMAGESIZE_Default);
+	virtual void create_image_file(Byte_Value size = IMAGESIZE_Default);
 	virtual void TearDown();
 
 public:
@@ -294,7 +217,7 @@ void SupportedFileSystemsTest::SetUp()
 }
 
 
-void SupportedFileSystemsTest::extra_setup(Byte_Value size)
+void SupportedFileSystemsTest::create_image_file(Byte_Value size)
 {
 	// Create new image file to work with.
 	unlink(s_image_name);
@@ -391,12 +314,7 @@ const std::string SupportedFileSystemsTest::create_loopdev(const std::string& im
 		return "";
 	}
 
-	// Strip trailing New Line.
-	size_t len = output.length();
-	if (len > 0 && output[len-1] == '\n')
-		output.resize(len-1);
-
-	return output;
+	return Utils::trim_trailing_new_line(output);
 }
 
 
@@ -485,7 +403,7 @@ TEST_P(SupportedFileSystemsTest, Create)
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 
-	extra_setup();
+	create_image_file();
 	// Call create, check for success and print operation details on failure.
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 }
@@ -495,11 +413,10 @@ TEST_P(SupportedFileSystemsTest, CreateAndReadUsage)
 {
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
 	SKIP_IF_FS_DOESNT_SUPPORT(read);
-	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_BTRFS);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
-	extra_setup();
+	create_image_file();
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
 	reload_partition();
@@ -525,12 +442,11 @@ TEST_P(SupportedFileSystemsTest, CreateAndReadLabel)
 {
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
 	SKIP_IF_FS_DOESNT_SUPPORT(read_label);
-	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_BTRFS);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
 	const char* fs_label = "TEST_LABEL";
-	extra_setup();
+	create_image_file();
 	m_partition.set_filesystem_label(fs_label);
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
@@ -548,11 +464,10 @@ TEST_P(SupportedFileSystemsTest, CreateAndReadUUID)
 {
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
 	SKIP_IF_FS_DOESNT_SUPPORT(read_uuid);
-	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_BTRFS);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
-	extra_setup();
+	create_image_file();
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
 	if (m_fstype == FS_JFS)
@@ -566,6 +481,7 @@ TEST_P(SupportedFileSystemsTest, CreateAndReadUUID)
 	// Test reading the UUID is successful.
 	reload_partition();
 	m_fs_object->read_uuid(m_partition);
+	IGNORE_REISER4_NULL_UUID();
 	EXPECT_GE(m_partition.uuid.size(), 9U);
 
 	// Test messages from read operation are empty or print them.
@@ -580,11 +496,15 @@ TEST_P(SupportedFileSystemsTest, CreateAndWriteLabel)
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
-	extra_setup();
+	create_image_file();
 	m_partition.set_filesystem_label("FIRST");
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
-	// Test writing a label is successful.
+	// Test clearing the label is successful.
+	m_partition.set_filesystem_label("");
+	ASSERT_TRUE(m_fs_object->write_label(m_partition, m_operation_detail)) << m_operation_detail;
+
+	// Test writing the label is successful.
 	m_partition.set_filesystem_label("SECOND");
 	ASSERT_TRUE(m_fs_object->write_label(m_partition, m_operation_detail)) << m_operation_detail;
 }
@@ -597,7 +517,7 @@ TEST_P(SupportedFileSystemsTest, CreateAndWriteUUID)
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
-	extra_setup();
+	create_image_file();
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
 	// Test writing a new random UUID is successful.
@@ -605,6 +525,32 @@ TEST_P(SupportedFileSystemsTest, CreateAndWriteUUID)
 }
 
 
+TEST_P(SupportedFileSystemsTest, CreateAndWriteUUIDAndReadLabel)
+{
+	SKIP_IF_FS_DOESNT_SUPPORT(create);
+	SKIP_IF_FS_DOESNT_SUPPORT(write_uuid);
+	SKIP_IF_FS_DOESNT_SUPPORT(read_label);
+	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
+	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
+
+	const char* fs_label = "TEST_LABEL";
+	create_image_file();
+	m_partition.set_filesystem_label(fs_label);
+	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
+
+	// Test writing a new random UUID is successful.
+	ASSERT_TRUE(m_fs_object->write_uuid(m_partition, m_operation_detail)) << m_operation_detail;
+
+	// Test reading the label is successful and it hasn't changed.
+	reload_partition();
+	m_fs_object->read_label(m_partition);
+	EXPECT_STREQ(fs_label, m_partition.get_filesystem_label().c_str());
+
+	// Test messages from read operation are empty or print them.
+	EXPECT_TRUE(m_partition.get_messages().empty()) << m_partition;
+}
+
+
 TEST_P(SupportedFileSystemsTest, CreateAndCheck)
 {
 	SKIP_IF_FS_DOESNT_SUPPORT(create);
@@ -612,7 +558,7 @@ TEST_P(SupportedFileSystemsTest, CreateAndCheck)
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_TEST_DISABLED_FOR_FS(FS_MINIX);  // FIXME: Enable when util-linux >= 2.27 is available everywhere
 
-	extra_setup();
+	create_image_file();
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
 	// Test checking the file system is successful.
@@ -626,7 +572,7 @@ TEST_P(SupportedFileSystemsTest, CreateAndRemove)
 	SKIP_IF_FS_DOESNT_SUPPORT(remove);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 
-	extra_setup();
+	create_image_file();
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
 	// Test removing the file system is successful.  Note that most file systems don't
@@ -645,7 +591,7 @@ TEST_P(SupportedFileSystemsTest, CreateAndGrow)
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_XFS);
 
-	extra_setup(IMAGESIZE_Default);
+	create_image_file(IMAGESIZE_Default);
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
 	// Test growing the file system is successful.
@@ -663,7 +609,7 @@ TEST_P(SupportedFileSystemsTest, CreateAndShrink)
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_LVM2_PV);
 	SKIP_IF_NOT_ROOT_FOR_REQUIRED_LOOPDEV_FOR_FS(FS_NILFS2);
 
-	extra_setup(IMAGESIZE_Larger);
+	create_image_file(IMAGESIZE_Larger);
 	ASSERT_TRUE(m_fs_object->create(m_partition, m_operation_detail)) << m_operation_detail;
 
 	// Test shrinking the file system is successful.
@@ -685,38 +631,6 @@ INSTANTIATE_TEST_CASE_P(My,
 }  // namespace GParted
 
 
-// Re-execute current executable using xvfb-run so that it provides a virtual X11 display.
-void exec_using_xvfb_run(int argc, char** argv)
-{
-	// argc+2 = Space for "xvfb-run" command, existing argc strings plus NULL pointer.
-	size_t size = sizeof(char*) * (argc+2);
-	char** new_argv = (char**)malloc(size);
-	if (new_argv == NULL)
-	{
-		fprintf(stderr, "Failed to allocate %lu bytes of memory.  errno=%d,%s\n",
-			(unsigned long)size, errno, strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-
-	new_argv[0] = strdup("xvfb-run");
-	if (new_argv[0] == NULL)
-	{
-		fprintf(stderr, "Failed to allocate %lu bytes of memory.  errno=%d,%s\n",
-		        (unsigned long)strlen(new_argv[0])+1, errno, strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-
-	// Copy argv pointers including final NULL pointer.
-	for (unsigned int i = 0; i <= (unsigned)argc; i++)
-		new_argv[i+1] = argv[i];
-
-	execvp(new_argv[0], new_argv);
-	fprintf(stderr, "Failed to execute '%s %s ...'.  errno=%d,%s\n", new_argv[0], new_argv[1],
-		errno, strerror(errno));
-	exit(EXIT_FAILURE);
-}
-
-
 // Custom Google Test main().
 // Reference:
 // *   Google Test, Primer, Writing the main() function
@@ -724,14 +638,7 @@ void exec_using_xvfb_run(int argc, char** argv)
 int main(int argc, char** argv)
 {
 	printf("Running main() from %s\n", __FILE__);
-
-	const char* display = getenv("DISPLAY");
-	if (display == NULL)
-	{
-		printf("DISPLAY environment variable unset.  Executing 'xvfb-run %s ...'\n", argv[0]);
-		exec_using_xvfb_run(argc, argv);
-	}
-	printf("DISPLAY=\"%s\"\n", display);
+	GParted::ensure_x11_display(argc, argv);
 
 	// Initialise threading in GParted to allow FileSystem interface classes to
 	// successfully use Utils:: and Filesystem::execute_command().  Must be before
